@@ -97,15 +97,22 @@ typedef struct {
 						   GList *item_uris,
 						   GdkPoint *relative_item_points,
 						   const char *target_uri,
-						   int copy_action,
+						   GdkDragAction action,
 						   int x,
 						   int y);
 	void	     (* handle_url)		  (NautilusIconContainer *container,
 						   char *url,
+						   GdkDragAction action,
 						   int x,
 						   int y);
 	void	     (* handle_uri_list)    	  (NautilusIconContainer *container,
 						   char *uri_list,
+						   GdkDragAction action,
+						   int x,
+						   int y);
+	void	     (* handle_text)		  (NautilusIconContainer *container,
+						   char *text,
+						   GdkDragAction action,
 						   int x,
 						   int y);
 
@@ -197,7 +204,8 @@ GtkWidget *       nautilus_icon_container_new                           (void);
 /* adding, removing, and managing icons */
 void              nautilus_icon_container_clear                         (NautilusIconContainer  *view);
 gboolean          nautilus_icon_container_add                           (NautilusIconContainer  *view,
-									 NautilusIconData       *data);
+									 NautilusIconData       *data,
+									 gboolean                has_lazy_position);
 void              nautilus_icon_container_layout_now                    (NautilusIconContainer *container);
 gboolean          nautilus_icon_container_remove                        (NautilusIconContainer  *view,
 									 NautilusIconData       *data);
