@@ -57,6 +57,7 @@ typedef struct {
 	/* Whether a monitor was set on this icon. */
 	eel_boolean_bit is_monitored : 1;
 
+	eel_boolean_bit has_lazy_position : 1;
 } NautilusIcon;
 
 
@@ -159,7 +160,6 @@ struct NautilusIconContainerDetails {
 	DragState drag_state;
 	gboolean drag_started;
 	StretchState stretch_start;
-	int context_menu_timeout_id;
 
 	gboolean icon_selected_on_button_down;
 	NautilusIcon *double_click_icon[2]; /* Both clicks in a double click need to be on the same icon */
