@@ -33,10 +33,25 @@
 #include <eel/eel-glib-extensions.h>
 #include <gtk/gtktreednd.h>
 #include <gtk/gtktreesortable.h>
+#include <libgnome/gnome-i18n.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-dnd.h>
 #include <gsequence/gsequence.h>
 
+enum {
+	SUBDIRECTORY_UNLOADED,
+	LAST_SIGNAL
+};
+
+/* msec delay after Loading... dummy row turns into (empty) */
+#define LOADING_TO_EMPTY_DELAY 100
+
+static guint list_model_signals[LAST_SIGNAL] = { 0 };
+
+static int fm_list_model_file_entry_compare_func (gconstpointer a,
+				       gconstpointer b,
+				       gpointer      user_data);
+
 static int fm_list_model_compare_func (gconstpointer a,
 				       gconstpointer b,
 				       gpointer      user_data);
@@ -67,19 +82,42 @@ typedef struct {
 	GList *path_list;
 } DragDataGetInfo;
 
+typedef struct FileEntry FileEntry;
+
+struct FileEntry {
+	NautilusFile *file;
+	NautilusDirectory *subdirectory;
+	FileEntry *parent;
+	GSequence *files;
+	GSequencePtr ptr;
+	guint loaded : 1;
+};
+
 static const GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
 	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
-	{ NAUTILUS_ICON_DND_TEXT_TYPE, 0, NAUTILUS_ICON_DND_TEXT }
 };
 
 static GtkTargetList *drag_target_list = NULL;
 
+static void
+file_entry_free (FileEntry *file_entry)
+{
+	nautilus_file_unref (file_entry->file);
+	if (file_entry->subdirectory != NULL) {
+		nautilus_directory_unref (file_entry->subdirectory);
+	}
+	if (file_entry->files != NULL) {
+		g_sequence_free (file_entry->files);
+	}
+	g_free (file_entry);
+}
+
 static guint
 fm_list_model_get_flags (GtkTreeModel *tree_model)
 {
-	return GTK_TREE_MODEL_ITERS_PERSIST | GTK_TREE_MODEL_LIST_ONLY;
+	return GTK_TREE_MODEL_ITERS_PERSIST;
 }
 
 static int
@@ -94,6 +132,8 @@ fm_list_model_get_column_type (GtkTreeModel *tree_model, int index)
 	switch (index) {
 	case FM_LIST_MODEL_FILE_COLUMN:
 		return NAUTILUS_TYPE_FILE;
+	case FM_LIST_MODEL_SUBDIRECTORY_COLUMN:
+		return NAUTILUS_TYPE_DIRECTORY;
 	case FM_LIST_MODEL_SMALLEST_ICON_COLUMN:
 	case FM_LIST_MODEL_SMALLER_ICON_COLUMN:
 	case FM_LIST_MODEL_SMALL_ICON_COLUMN:
@@ -124,19 +164,27 @@ static gboolean
 fm_list_model_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreePath *path)
 {
 	FMListModel *model;
+	GSequence *files;
 	GSequencePtr ptr;
-	int i;
+	FileEntry *file_entry;
+	int i, d;
 	
 	model = (FMListModel *)tree_model;
+	ptr = NULL;
+	
+	files = model->details->files;
+	for (d = 0; d < gtk_tree_path_get_depth (path); d++) {
+		i = gtk_tree_path_get_indices (path)[d];
 
-	i = gtk_tree_path_get_indices (path)[0];
+		if (files == NULL || i >= g_sequence_get_length (files)) {
+			return FALSE;
+		}
 
-	if (i >= g_sequence_get_length (model->details->files)) {
-		return FALSE;
+		ptr = g_sequence_get_ptr_at_pos (files, i);
+		file_entry = g_sequence_ptr_get_data (ptr);
+		files = file_entry->files;
 	}
 
-	ptr = g_sequence_get_ptr_at_pos (model->details->files, i);
-
 	iter->stamp = model->details->stamp;
 	iter->user_data = ptr;
 
@@ -150,6 +198,9 @@ fm_list_model_get_path (GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
 	GtkTreePath *path;
 	FMListModel *model;
+	GSequencePtr ptr;
+	FileEntry *file_entry;
+
 
 	model = (FMListModel *)tree_model;
 	
@@ -161,7 +212,16 @@ fm_list_model_get_path (GtkTreeModel *tree_model, GtkTreeIter *iter)
 	}
 	
 	path = gtk_tree_path_new ();
-	gtk_tree_path_append_index (path, g_sequence_ptr_get_position (iter->user_data));
+	ptr = iter->user_data;
+	while (ptr != NULL) {
+		gtk_tree_path_prepend_index (path, g_sequence_ptr_get_position (ptr));
+		file_entry = g_sequence_ptr_get_data (ptr);
+		if (file_entry->parent != NULL) {
+			ptr = file_entry->parent->ptr;
+		} else {
+			ptr = NULL;
+		}
+	}
 
 	return path;
 }
@@ -170,6 +230,7 @@ static void
 fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column, GValue *value)
 {
 	FMListModel *model;
+	FileEntry *file_entry;
 	NautilusFile *file;
 	char *str;
 	GdkPixbuf *icon;
@@ -185,7 +246,8 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	g_return_if_fail (model->details->stamp == iter->stamp);
 	g_return_if_fail (!g_sequence_ptr_is_end (iter->user_data));
 
-	file = g_sequence_ptr_get_data (iter->user_data);
+	file_entry = g_sequence_ptr_get_data (iter->user_data);
+	file = file_entry->file;
 	
 	switch (column) {
 	case FM_LIST_MODEL_FILE_COLUMN:
@@ -193,6 +255,11 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 
 		g_value_set_object (value, file);
 		break;
+	case FM_LIST_MODEL_SUBDIRECTORY_COLUMN:
+		g_value_init (value, NAUTILUS_TYPE_DIRECTORY);
+
+		g_value_set_object (value, file_entry->subdirectory);
+		break;
 	case FM_LIST_MODEL_SMALLEST_ICON_COLUMN:
 	case FM_LIST_MODEL_SMALLER_ICON_COLUMN:
 	case FM_LIST_MODEL_SMALL_ICON_COLUMN:
@@ -202,36 +269,38 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	case FM_LIST_MODEL_LARGEST_ICON_COLUMN:
 		g_value_init (value, GDK_TYPE_PIXBUF);
 
-		zoom_level = fm_list_model_get_zoom_level_from_column_id (column);
-		icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
+		if (file != NULL) {
+			zoom_level = fm_list_model_get_zoom_level_from_column_id (column);
+			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
 
-		modifier = NULL;
-		if (model->details->drag_view != NULL) {
-			GtkTreePath *path_a, *path_b;
-			
-			gtk_tree_view_get_drag_dest_row (model->details->drag_view,
-							 &path_a,
-							 NULL);
-			if (path_a != NULL) {
-				path_b = gtk_tree_model_get_path (tree_model, iter);
+			modifier = NULL;
+			if (model->details->drag_view != NULL) {
+				GtkTreePath *path_a, *path_b;
+				
+				gtk_tree_view_get_drag_dest_row (model->details->drag_view,
+								 &path_a,
+								 NULL);
+				if (path_a != NULL) {
+					path_b = gtk_tree_model_get_path (tree_model, iter);
 
-				if (gtk_tree_path_compare (path_a, path_b) == 0) {
-					modifier = "accept";
+					if (gtk_tree_path_compare (path_a, path_b) == 0) {
+						modifier = "accept";
+					}
+						
+					gtk_tree_path_free (path_a);
+					gtk_tree_path_free (path_b);
 				}
-					
-				gtk_tree_path_free (path_a);
-				gtk_tree_path_free (path_b);
 			}
-		}
-		
-		if (nautilus_file_has_open_window (file)) {
-			modifier = "visiting";
-		}
-		
-		icon = nautilus_icon_factory_get_pixbuf_for_file_force_size (file, modifier, icon_size);
+			
+			if (nautilus_file_has_open_window (file)) {
+				modifier = "visiting";
+			}
+			
+			icon = nautilus_icon_factory_get_pixbuf_for_file_force_size (file, modifier, icon_size);
 
-		g_value_set_object (value, icon);
-		g_object_unref (icon);
+			g_value_set_object (value, icon);
+			g_object_unref (icon);
+		}
 		break;
 	case FM_LIST_MODEL_SMALLEST_EMBLEM_COLUMN:
 	case FM_LIST_MODEL_SMALLER_EMBLEM_COLUMN:
@@ -242,39 +311,41 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	case FM_LIST_MODEL_LARGEST_EMBLEM_COLUMN:
 		g_value_init (value, GDK_TYPE_PIXBUF);
 
-		parent_file = nautilus_file_get_parent (file);
-		emblems_to_ignore = eel_string_list_new_from_string (NAUTILUS_FILE_EMBLEM_NAME_TRASH, TRUE);
-		if (parent_file) {
-			if (!nautilus_file_can_write (parent_file)) {
-				eel_string_list_prepend (emblems_to_ignore, NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE);
+		if (file != NULL) {
+			parent_file = nautilus_file_get_parent (file);
+			emblems_to_ignore = eel_string_list_new_from_string (NAUTILUS_FILE_EMBLEM_NAME_TRASH, TRUE);
+			if (parent_file) {
+				if (!nautilus_file_can_write (parent_file)) {
+					eel_string_list_prepend (emblems_to_ignore, NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE);
+				}
+				nautilus_file_unref (parent_file);
 			}
-			nautilus_file_unref (parent_file);
-		}
-		emblem_icons = nautilus_icon_factory_get_emblem_icons_for_file (file, emblems_to_ignore);
-		eel_string_list_free (emblems_to_ignore);
+			emblem_icons = nautilus_icon_factory_get_emblem_icons_for_file (file, emblems_to_ignore);
+			eel_string_list_free (emblems_to_ignore);
 
-		if (emblem_icons != NULL) {
-			zoom_level = fm_list_model_get_zoom_level_from_emblem_column_id (column);
-			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
-			icon = nautilus_icon_factory_get_pixbuf_for_icon_force_size (
-				emblem_icons->data, NULL, icon_size,
-				NULL, NULL, FALSE, NULL);
-			eel_g_list_free_deep (emblem_icons);
-	
-			g_value_set_object (value, icon);
+			if (emblem_icons != NULL) {
+				zoom_level = fm_list_model_get_zoom_level_from_emblem_column_id (column);
+				icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
+				icon = nautilus_icon_factory_get_pixbuf_for_icon_force_size (
+					emblem_icons->data, NULL, icon_size,
+					NULL, NULL, FALSE, NULL);
+				eel_g_list_free_deep (emblem_icons);
+		
+				g_value_set_object (value, icon);
 
-			if (icon != NULL) { 
-				g_object_unref (icon);
+				if (icon != NULL) { 
+					g_object_unref (icon);
+				}
 			}
 		}
 		break;
 	case FM_LIST_MODEL_FILE_NAME_IS_EDITABLE_COLUMN:
 		g_value_init (value, G_TYPE_BOOLEAN);
 		
-                g_value_set_boolean (value, nautilus_file_can_rename (file));
+                g_value_set_boolean (value, file != NULL && nautilus_file_can_rename (file));
                 break;
  	default:
-		if (column >= FM_LIST_MODEL_NUM_COLUMNS || column < FM_LIST_MODEL_NUM_COLUMNS + model->details->columns->len) {
+ 		if (column >= FM_LIST_MODEL_NUM_COLUMNS || column < FM_LIST_MODEL_NUM_COLUMNS + model->details->columns->len) {
 			NautilusColumn *nautilus_column;
 			char *attribute;
 			nautilus_column = model->details->columns->pdata[column - FM_LIST_MODEL_NUM_COLUMNS];
@@ -283,9 +354,17 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 			g_object_get (nautilus_column, 
 				      "attribute", &attribute, 
 				      NULL);
-			str = nautilus_file_get_string_attribute_with_default (file, 
+			if (file != NULL) {
+				str = nautilus_file_get_string_attribute_with_default (file, 
 									       attribute);
-			g_value_set_string_take_ownership (value, str);
+				g_value_set_string_take_ownership (value, str);
+			} else if (!strcmp (attribute, "name")) {
+				if (file_entry->parent->loaded) {
+					g_value_set_string (value, _("(Empty)"));
+				} else {
+					g_value_set_string (value, _("Loading..."));
+				}
+			}
 			g_free (attribute);
 		} else {
 			g_assert_not_reached ();
@@ -311,19 +390,24 @@ static gboolean
 fm_list_model_iter_children (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreeIter *parent)
 {
 	FMListModel *model;
+	GSequence *files;
+	FileEntry *file_entry;
 
 	model = (FMListModel *)tree_model;
 	
-	if (parent != NULL) {
-		return FALSE;
+	if (parent == NULL) {
+		files = model->details->files;
+	} else {
+		file_entry = g_sequence_ptr_get_data (parent->user_data);
+		files = file_entry->files;
 	}
 
-	if (g_sequence_get_length (model->details->files) == 0) {
+	if (files == NULL || g_sequence_get_length (files) == 0) {
 		return FALSE;
 	}
-
+	
 	iter->stamp = model->details->stamp;
-	iter->user_data = g_sequence_get_begin_ptr (model->details->files);
+	iter->user_data = g_sequence_get_begin_ptr (files);
 
 	return TRUE;
 }
@@ -331,23 +415,34 @@ fm_list_model_iter_children (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTre
 static gboolean
 fm_list_model_iter_has_child (GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
-	return FALSE;
+	FileEntry *file_entry;
+	
+	if (iter == NULL) {
+		return !fm_list_model_is_empty (FM_LIST_MODEL (tree_model));
+	}
+
+	file_entry = g_sequence_ptr_get_data (iter->user_data);
+
+	return (file_entry->files != NULL && g_sequence_get_length (file_entry->files) > 0);
 }
 
 static int
 fm_list_model_iter_n_children (GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
 	FMListModel *model;
+	GSequence *files;
+	FileEntry *file_entry;
 
 	model = (FMListModel *)tree_model;
 
 	if (iter == NULL) {
-		return g_sequence_get_length (model->details->files);
+		files = model->details->files;
+	} else {
+		file_entry = g_sequence_ptr_get_data (iter->user_data);
+		files = file_entry->files;
 	}
 
-	g_return_val_if_fail (model->details->stamp == iter->stamp, -1);
-
-	return 0;
+	return g_sequence_get_length (files);
 }
 
 static gboolean
@@ -355,14 +450,19 @@ fm_list_model_iter_nth_child (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTr
 {
 	FMListModel *model;
 	GSequencePtr child;
+	GSequence *files;
+	FileEntry *file_entry;
 
 	model = (FMListModel *)tree_model;
 	
 	if (parent != NULL) {
-		return FALSE;
+		file_entry = g_sequence_ptr_get_data (parent->user_data);
+		files = file_entry->files;
+	} else {
+		files = model->details->files;
 	}
 
-	child = g_sequence_get_ptr_at_pos (model->details->files, n);
+	child = g_sequence_get_ptr_at_pos (files, n);
 
 	if (g_sequence_ptr_is_end (child)) {
 		return FALSE;
@@ -377,7 +477,21 @@ fm_list_model_iter_nth_child (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTr
 static gboolean
 fm_list_model_iter_parent (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreeIter *child)
 {
-	return FALSE;
+	FMListModel *model;
+	FileEntry *file_entry;
+	
+	model = (FMListModel *)tree_model;
+	
+	file_entry = g_sequence_ptr_get_data (child->user_data);
+	
+	if (file_entry->parent == NULL) {
+		return FALSE;
+	}
+
+	iter->stamp = model->details->stamp;
+	iter->user_data = file_entry->parent->ptr;
+	
+	return TRUE;
 }
 
 gboolean
@@ -392,7 +506,7 @@ fm_list_model_get_tree_iter_from_file (FMListModel *model, NautilusFile *file, G
 	}
 
 	g_assert (!g_sequence_ptr_is_end (ptr));
-	g_assert (g_sequence_ptr_get_data (ptr) == file);
+	g_assert (((FileEntry *)g_sequence_ptr_get_data (ptr))->file == file);
 	
 	if (iter != NULL) {
 		iter->stamp = model->details->stamp;
@@ -402,6 +516,35 @@ fm_list_model_get_tree_iter_from_file (FMListModel *model, NautilusFile *file, G
 	return TRUE;
 }
 
+static int
+fm_list_model_file_entry_compare_func (gconstpointer a,
+				       gconstpointer b,
+				       gpointer      user_data)
+{
+	FileEntry *file_entry1;
+	FileEntry *file_entry2;
+	FMListModel *model;
+	int result;
+
+	model = (FMListModel *)user_data;
+
+	file_entry1 = (FileEntry *)a;
+	file_entry2 = (FileEntry *)b;
+	
+	if (file_entry1->file != NULL && file_entry2->file != NULL) {
+		result = nautilus_file_compare_for_sort_by_attribute (file_entry1->file, file_entry2->file,
+							      model->details->sort_attribute,
+							      model->details->sort_directories_first,
+							      (model->details->order == GTK_SORT_DESCENDING));
+	} else if (file_entry1->file == NULL) {
+		return -1;
+	} else {
+		return 1;
+	}
+
+	return result;
+}
+
 static int
 fm_list_model_compare_func (gconstpointer a,
 			    gconstpointer b,
@@ -416,7 +559,7 @@ fm_list_model_compare_func (gconstpointer a,
 
 	file1 = (NautilusFile *)a;
 	file2 = (NautilusFile *)b;
-
+	
 	result = nautilus_file_compare_for_sort_by_attribute (file1, file2,
 							      model->details->sort_attribute,
 							      model->details->sort_directories_first,
@@ -426,16 +569,16 @@ fm_list_model_compare_func (gconstpointer a,
 }
 
 static void
-fm_list_model_sort (FMListModel *model)
+fm_list_model_sort_file_entries (FMListModel *model, GSequence *files, GtkTreePath *path)
 {
-	GSequence *files;
 	GSequencePtr *old_order;
-	GtkTreePath *path;
+	GtkTreeIter iter;
 	int *new_order;
 	int length;
 	int i;
+	FileEntry *file_entry;
+	gboolean has_iter;
 
-	files = model->details->files;
 	length = g_sequence_get_length (files);
 
 	if (length <= 1) {
@@ -446,12 +589,19 @@ fm_list_model_sort (FMListModel *model)
 	old_order = g_new (GSequencePtr, length);
 	for (i = 0; i < length; ++i) {
 		GSequencePtr ptr = g_sequence_get_ptr_at_pos (files, i);
+		
+		file_entry = g_sequence_ptr_get_data (ptr);
+		if (file_entry->files != NULL) {
+			gtk_tree_path_append_index (path, i);
+			fm_list_model_sort_file_entries (model, file_entry->files, path);
+			gtk_tree_path_up (path);
+		}
 
 		old_order[i] = ptr;
 	}
 
 	/* sort */
-	g_sequence_sort (files, fm_list_model_compare_func, model);
+	g_sequence_sort (files, fm_list_model_file_entry_compare_func, model);
 
 	/* generate new order */
 	new_order = g_new (int, length);
@@ -461,17 +611,36 @@ fm_list_model_sort (FMListModel *model)
 	}
 
 	/* Let the world know about our new order */
-	path = gtk_tree_path_new ();
 
 	g_assert (new_order != NULL);
-	gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model),
-				       path, NULL, new_order);
 
-	gtk_tree_path_free (path);
+	has_iter = FALSE;
+	if (gtk_tree_path_get_depth (path) != 0) {
+		gboolean get_iter_result;
+		has_iter = TRUE;
+		get_iter_result = gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path);
+		g_assert (get_iter_result);
+	}
+
+	gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model),
+				       path, has_iter ? &iter : NULL, new_order);
+
 	g_free (old_order);
 	g_free (new_order);
 }
 
+static void
+fm_list_model_sort (FMListModel *model)
+{
+	GtkTreePath *path;
+
+	path = gtk_tree_path_new ();
+
+	fm_list_model_sort_file_entries (model, model->details->files, path);
+
+	gtk_tree_path_free (path);
+}
+
 void
 fm_list_model_sort_files (FMListModel *model, GList **files)
 {
@@ -607,7 +776,8 @@ fm_list_model_multi_drag_data_get (EggTreeMultiDragSource *drag_source,
 	if (!drag_target_list) {
 		drag_target_list = gtk_target_list_new 
 			(drag_types, G_N_ELEMENTS (drag_types));
-
+		gtk_target_list_add_text_targets (
+			drag_target_list, NAUTILUS_ICON_DND_TEXT);
 	}
 
 	if (gtk_target_list_find (drag_target_list,
@@ -632,31 +802,129 @@ fm_list_model_multi_drag_data_delete (EggTreeMultiDragSource *drag_source, GList
 	return TRUE;
 }
 
-void
+static void
+add_dummy_row (FMListModel *model, FileEntry *parent_entry)
+{
+	FileEntry *dummy_file_entry;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	
+	dummy_file_entry = g_new0 (FileEntry, 1);
+	dummy_file_entry->parent = parent_entry;
+	dummy_file_entry->ptr = g_sequence_insert_sorted (parent_entry->files, dummy_file_entry,
+							  fm_list_model_file_entry_compare_func, model);
+	iter.stamp = model->details->stamp;
+	iter.user_data = dummy_file_entry->ptr;
+	
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+	gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
+	gtk_tree_path_free (path);
+}
+
+gboolean
 fm_list_model_add_file (FMListModel *model, NautilusFile *file)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	GSequencePtr new_ptr;
-	
-	/* We may only add each file once. */
-	if (fm_list_model_get_tree_iter_from_file (model, file, NULL)) {
-		return;
+	FileEntry *file_entry;
+	NautilusFile *parent_file;
+	GSequencePtr ptr, parent_ptr;
+	GSequence *files;
+	gboolean replace_dummy;
+
+	/* We may only add each file once, in one dir. */
+	ptr = g_hash_table_lookup (model->details->reverse_map, file);
+	if (ptr != NULL) {
+		file_entry = g_sequence_ptr_get_data (ptr);
+		
+		parent_file = nautilus_file_get_parent (file);
+		if (parent_file == NULL) {
+			return FALSE;
+		}
+		parent_ptr = g_hash_table_lookup (model->details->reverse_map,
+						  parent_file);
+		nautilus_file_unref (parent_file);
+		if (/* toplevel parent */
+		    (file_entry->parent == NULL && parent_ptr == NULL) ||
+		    /* same in-tree parent */
+		    (file_entry->parent != NULL && parent_ptr != NULL &&
+		     file_entry->parent->ptr == parent_ptr) ) {
+			/* Don't add twice in same place */
+			return FALSE; 
+		}
+
+		/* file has a new parent, due to a move
+		 * this happens because a move results in a
+		 * CHANGE event and an ADD event, and
+		 * if the target directory of the move is
+		 * added to the FMDirectoryView
+		 * then we won't remove the changed file.
+		 */
+		fm_list_model_remove_file (model, file);
+
+		/* Now add it back again in the new place. */
 	}
 
 	nautilus_file_ref (file);
 	
-	new_ptr = g_sequence_insert_sorted (model->details->files, file,
-					    fm_list_model_compare_func, model);
+	file_entry = g_new0 (FileEntry, 1);
+	file_entry->file = file;
+	file_entry->parent = NULL;
+	file_entry->subdirectory = NULL;
+	file_entry->files = NULL;
+	
+	files = model->details->files;
+	
+	replace_dummy = FALSE;
 
-	g_hash_table_insert (model->details->reverse_map, file, new_ptr);
+	parent_file = nautilus_file_get_parent (file);
+	if (parent_file != NULL) {
+		parent_ptr = g_hash_table_lookup (model->details->reverse_map,
+						  parent_file);
+		nautilus_file_unref (parent_file);
+		if (parent_ptr != NULL) {
+			file_entry->parent = g_sequence_ptr_get_data (parent_ptr);
+			files = file_entry->parent->files;
+			if (g_sequence_get_length (files) == 1) {
+				GSequencePtr dummy_ptr = g_sequence_get_ptr_at_pos (files, 0);
+				FileEntry *dummy_entry = g_sequence_ptr_get_data (dummy_ptr);
+				if (dummy_entry->file == NULL) {
+					/* replace the dummy loading entry */
+					model->details->stamp++;
+					g_sequence_remove (dummy_ptr);
+					
+					replace_dummy = TRUE;
+				}
+			}
+		}
+	}
+	
+	file_entry->ptr = g_sequence_insert_sorted (files, file_entry,
+					    fm_list_model_file_entry_compare_func, model);
+
+	g_hash_table_insert (model->details->reverse_map, file, file_entry->ptr);
 	
 	iter.stamp = model->details->stamp;
-	iter.user_data = new_ptr;
+	iter.user_data = file_entry->ptr;
 
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
-	gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
+	if (replace_dummy) {
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+	} else {
+		gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
+	}
+
+	if (nautilus_file_is_directory (file)) {
+		file_entry->files = g_sequence_new ((GDestroyNotify)file_entry_free);
+
+		add_dummy_row (model, file_entry);
+
+		gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model),
+						      path, &iter);
+	}
 	gtk_tree_path_free (path);
+	
+	return TRUE;
 }
 
 void
@@ -671,7 +939,7 @@ fm_list_model_file_changed (FMListModel *model, NautilusFile *file)
 		return;
 	}
 
-	g_sequence_ptr_sort_changed (ptr, fm_list_model_compare_func, model);
+	g_sequence_ptr_sort_changed (ptr, fm_list_model_file_entry_compare_func, model);
 
 	if (!fm_list_model_get_tree_iter_from_file (model, file, &iter)) {
 		return;
@@ -694,23 +962,73 @@ fm_list_model_get_length (FMListModel *model)
 	return g_sequence_get_length (model->details->files);
 }
 
-
 static void
 fm_list_model_remove (FMListModel *model, GtkTreeIter *iter)
 {
-	GSequencePtr ptr;
+	GSequencePtr ptr, child_ptr;
+	FileEntry *file_entry, *child_file_entry, *parent_file_entry;
 	GtkTreePath *path;
+	GtkTreeIter parent_iter;
 
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), iter);
 	ptr = iter->user_data;
-
-	g_hash_table_remove (model->details->reverse_map, g_sequence_ptr_get_data (ptr));
-	g_sequence_remove (ptr);
-
-	model->details->stamp++;
+	file_entry = g_sequence_ptr_get_data (ptr);
 	
+	if (file_entry->files != NULL) {
+		while (g_sequence_get_length (file_entry->files) > 0) {
+			child_ptr = g_sequence_get_begin_ptr (file_entry->files);
+			child_file_entry = g_sequence_ptr_get_data (child_ptr);
+			if (child_file_entry->file != NULL) {
+				fm_list_model_remove_file (model,
+							child_file_entry->file);
+			} else {
+				path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), iter);
+				gtk_tree_path_append_index (path, 0);
+				model->details->stamp++;
+				g_sequence_remove (child_ptr);
+				gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
+				gtk_tree_path_free (path);
+			}
+			
+			/* the parent iter didn't actually change */
+			iter->stamp = model->details->stamp;
+		}
+			
+	}
+
+	g_hash_table_remove (model->details->reverse_map, file_entry->file);
+
+	parent_file_entry = file_entry->parent;
+	if (parent_file_entry && g_sequence_get_length (parent_file_entry->files) == 1 &&
+	    file_entry->file != NULL) {
+		/* this is the last non-dummy child, add a dummy node */
+		/* We need to do this before removing the last file to avoid
+		 * collapsing the row.
+		 */
+		add_dummy_row (model, parent_file_entry);
+	}
+
+	if (file_entry->subdirectory != NULL) {
+		g_signal_emit (model,
+			       list_model_signals[SUBDIRECTORY_UNLOADED], 0,
+			       file_entry->subdirectory);
+	}
+	
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), iter);
+	
+	g_sequence_remove (ptr);
+	model->details->stamp++;
 	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
+	
 	gtk_tree_path_free (path);
+
+	if (parent_file_entry && g_sequence_get_length (parent_file_entry->files) == 0) {
+		parent_iter.stamp = model->details->stamp;
+		parent_iter.user_data = parent_file_entry->ptr;
+		path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &parent_iter);
+		gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model),
+						      path, &parent_iter);
+		gtk_tree_path_free (path);
+	}
 }
 
 void
@@ -723,18 +1041,31 @@ fm_list_model_remove_file (FMListModel *model, NautilusFile *file)
 	}
 }
 
+static void
+fm_list_model_clear_directory (FMListModel *model, GSequence *files)
+{
+	GtkTreeIter iter;
+	FileEntry *file_entry;
+
+	while (g_sequence_get_length (files) > 0) {
+		iter.user_data = g_sequence_get_begin_ptr (files);
+
+		file_entry = g_sequence_ptr_get_data (iter.user_data);
+		if (file_entry->files != NULL) {
+			fm_list_model_clear_directory (model, file_entry->files);
+		}
+		
+		iter.stamp = model->details->stamp;
+		fm_list_model_remove (model, &iter);
+	}
+}
+
 void
 fm_list_model_clear (FMListModel *model)
 {
-	GtkTreeIter iter;
-
 	g_return_if_fail (model != NULL);
 
-	while (g_sequence_get_length (model->details->files) > 0) {
-		iter.stamp = model->details->stamp;
-		iter.user_data = g_sequence_get_begin_ptr (model->details->files);
-		fm_list_model_remove (model, &iter);
-	}
+	fm_list_model_clear_directory (model, model->details->files);
 }
 
 NautilusFile *
@@ -754,6 +1085,69 @@ fm_list_model_file_for_path (FMListModel *model, GtkTreePath *path)
 	return file;
 }
 
+gboolean
+fm_list_model_load_subdirectory (FMListModel *model, GtkTreePath *path, NautilusDirectory **directory)
+{
+	GtkTreeIter iter;
+	FileEntry *file_entry;
+	
+	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (model), &iter, path)) {
+		return FALSE;
+	}
+
+	file_entry = g_sequence_ptr_get_data (iter.user_data);
+	if (file_entry->file == NULL ||
+	    file_entry->subdirectory != NULL) {
+		return FALSE;
+	}
+
+	file_entry->subdirectory = nautilus_directory_get_for_file (file_entry->file);
+	nautilus_directory_ref (file_entry->subdirectory);
+	*directory = file_entry->subdirectory;
+	
+	return TRUE;
+}
+
+/* removes all children of the subfolder and unloads the subdirectory */
+void
+fm_list_model_unload_subdirectory (FMListModel *model, GtkTreeIter *iter)
+{
+	GSequencePtr child_ptr;
+	FileEntry *file_entry, *child_file_entry;
+	
+	file_entry = g_sequence_ptr_get_data (iter->user_data);
+	if (file_entry->file == NULL ||
+	    file_entry->subdirectory == NULL) {
+		return;
+	}
+
+	file_entry->loaded = 0;
+	
+	/* Remove all children */
+	while (g_sequence_get_length (file_entry->files) > 0) {
+		child_ptr = g_sequence_get_begin_ptr (file_entry->files);
+		child_file_entry = g_sequence_ptr_get_data (child_ptr);
+		if (child_file_entry->file == NULL) {
+			/* Don't delete the dummy node */
+			break;
+		} else {
+			fm_list_model_remove_file (model,
+						   child_file_entry->file);
+		}
+	}
+
+	/* Emit unload signal */
+	g_signal_emit (model,
+		       list_model_signals[SUBDIRECTORY_UNLOADED], 0,
+		       file_entry->subdirectory);
+
+	/* actually unload */
+	nautilus_directory_unref (file_entry->subdirectory);
+	file_entry->subdirectory = NULL;
+}
+
+
+
 void
 fm_list_model_set_should_sort_directories_first (FMListModel *model, gboolean sort_directories_first)
 {
@@ -1017,7 +1411,7 @@ static void
 fm_list_model_init (FMListModel *model)
 {
 	model->details = g_new0 (FMListModelDetails, 1);
-	model->details->files = g_sequence_new ((GDestroyNotify)nautilus_file_unref);
+	model->details->files = g_sequence_new ((GDestroyNotify)file_entry_free);
 	model->details->reverse_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 	model->details->stamp = g_random_int ();
 	model->details->sort_attribute = NULL;
@@ -1034,6 +1428,16 @@ fm_list_model_class_init (FMListModelClass *klass)
 
 	object_class->finalize = fm_list_model_finalize;
 	object_class->dispose = fm_list_model_dispose;
+
+      list_model_signals[SUBDIRECTORY_UNLOADED] =
+        g_signal_new ("subdirectory_unloaded",
+                      FM_TYPE_LIST_MODEL,
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (FMListModelClass, subdirectory_unloaded),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__OBJECT,
+                      G_TYPE_NONE, 1,
+                      NAUTILUS_TYPE_DIRECTORY);
 }
 
 static void
@@ -1119,3 +1523,83 @@ fm_list_model_get_type (void)
 	
 	return object_type;
 }
+
+struct ChangeDummyData {
+	FMListModel *model;
+	NautilusFile *file;
+};
+
+static gboolean
+change_dummy_row_callback (gpointer callback_data)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	FileEntry *file_entry, *dummy_entry;
+	GSequencePtr parent_ptr, dummy_ptr;
+	FMListModel *model;
+	GSequence *files;
+	
+	struct ChangeDummyData *data;
+	data = callback_data;
+
+	if (data->model != NULL) {
+		model = data->model;
+		
+		parent_ptr = g_hash_table_lookup (model->details->reverse_map,
+						  data->file);
+		file_entry = g_sequence_ptr_get_data (parent_ptr);
+
+		file_entry->loaded = 1;
+		files = file_entry->files;
+	
+		if (g_sequence_get_length (files) == 1) {
+			dummy_ptr = g_sequence_get_ptr_at_pos (file_entry->files, 0);
+			dummy_entry = g_sequence_ptr_get_data (dummy_ptr);
+			if (dummy_entry->file == NULL) {
+				/* was the dummy file */
+				
+				iter.stamp = model->details->stamp;
+				iter.user_data = dummy_ptr;
+				
+				path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+				gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+				gtk_tree_path_free (path);
+			}
+		}
+	}
+
+	eel_remove_weak_pointer (&data->model);
+	nautilus_file_unref (data->file);
+	g_free (data);
+	
+	return FALSE;
+}
+
+void
+fm_list_model_subdirectory_done_loading (FMListModel *model, NautilusDirectory *directory)
+{
+	NautilusFile *parent_file;
+	GSequencePtr parent_ptr;
+	struct ChangeDummyData *data;
+
+	parent_file = nautilus_directory_get_corresponding_file (directory);
+	parent_ptr = g_hash_table_lookup (model->details->reverse_map,
+					  parent_file);
+	if (parent_ptr == NULL) {
+		nautilus_file_unref (parent_file);
+		return;
+	}
+
+	/* Delay slightly before sending the change even for the dummy
+	 * row, to avoid flicker from loading -> empty -> file if we
+	 * get files added quickly.
+	 */
+	data = g_new (struct ChangeDummyData, 1);
+	data->model = model;
+	data->file = nautilus_file_ref (parent_file);
+	eel_add_weak_pointer (&data->model);
+
+	g_timeout_add (LOADING_TO_EMPTY_DELAY, change_dummy_row_callback, data);
+
+	nautilus_file_unref (parent_file);
+}
