@@ -27,6 +27,7 @@
 #include "nautilus-file-management-properties.h"
 
 #include <string.h>
+#include <time.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
@@ -55,6 +56,7 @@
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ICON_VIEW_ZOOM_WIDGET "iconview_zoom_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_LIST_VIEW_ZOOM_WIDGET "listview_zoom_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET "sort_order_optionmenu"
+#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET "date_format_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_TEXT_WIDGET "preview_text_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_IMAGE_WIDGET "preview_image_optionmenu"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_SOUND_WIDGET "preview_sound_optionmenu"
@@ -64,6 +66,7 @@
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_FOLDERS_FIRST_WIDGET "sort_folders_first_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_COMPACT_LAYOUT_WIDGET "compact_layout_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_LABELS_BESIDE_ICONS_WIDGET "labels_beside_icons_checkbutton"
+#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET "always_use_browser_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET "trash_confirm_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_DELETE_WIDGET "trash_delete_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_OPEN_NEW_WINDOW_WIDGET "new_window_checkbutton"
@@ -73,13 +76,13 @@
 /* int enums */
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_THUMBNAIL_LIMIT_WIDGET "preview_image_size_optionmenu"
 
-static const char *default_view_values[] = {
+static const char * const default_view_values[] = {
 	"icon_view",
 	"list_view",
 	NULL
 };
 
-static const char *zoom_values[] = {
+static const char * const zoom_values[] = {
 	"smallest",
 	"smaller",
 	"small",
@@ -90,7 +93,7 @@ static const char *zoom_values[] = {
 	NULL
 };
 
-static const char *sort_order_values[] = {
+static const char * const sort_order_values[] = {
 	"name",
 	"size",
 	"type",
@@ -99,40 +102,47 @@ static const char *sort_order_values[] = {
 	NULL
 };
 
-static const char *preview_values[] = {
+static const char * const date_format_values[] = {
+	"locale",
+	"iso",
+	"informal",
+	NULL
+};
+
+static const char * const preview_values[] = {
 	"always",
 	"local_only",
 	"never",
 	NULL
 };
 
-static const char *click_behavior_components[] = {
+static const char * const click_behavior_components[] = {
 	"single_click_radiobutton",
 	"double_click_radiobutton",
 	NULL
 };
 
-static const char *click_behavior_values[] = {
+static const char * const click_behavior_values[] = {
 	"single",
 	"double",
 	NULL
 };
 
-static const char *executable_text_components[] = {
+static const char * const executable_text_components[] = {
 	"scripts_execute_radiobutton",
 	"scripts_view_radiobutton",
 	"scripts_confirm_radiobutton",
 	NULL
 };
 
-static const char *executable_text_values[] = {
+static const char * const executable_text_values[] = {
 	"launch",
 	"display",
 	"ask",
 	NULL
 };
 
-static int thumbnail_limit_values[] = {
+static const int thumbnail_limit_values[] = {
 	102400,
 	512000,
 	1048576,
@@ -144,7 +154,7 @@ static int thumbnail_limit_values[] = {
 	-1
 };
 
-static const char *icon_captions_components[] = {
+static const char * const icon_captions_components[] = {
 	"captions_0_optionmenu",
 	"captions_1_optionmenu",
 	"captions_2_optionmenu",
@@ -243,14 +253,16 @@ nautilus_file_management_properties_dialog_response_cb (GtkDialog *parent,
 			section = "gosnautilus-438";
 			break;
 		case 1:
-			section = "gosnautilus-57";
+			section = "gosnautilus-56";
 			break;
 		case 2:
 			section = "gosnautilus-439";
 			break;
 		case 3:
-			section = "gosnautilus-60";
+			section = "gosnautilus-490";
 			break;
+		case 4:
+			section = "gosnautilus-60";
 		}
 		preferences_show_help (GTK_WINDOW (parent), "user-guide.xml", section);
 	} else if (response_id == GTK_RESPONSE_CLOSE) {
@@ -450,6 +462,43 @@ nautilus_file_management_properties_dialog_setup_icon_caption_page (GladeXML *xm
 	update_icon_captions_from_gconf (xml_dialog);
 }
 
+static void
+create_date_format_menu (GladeXML *xml_dialog)
+{
+	GtkWidget *option_menu;
+	GtkWidget *menu;
+	GtkWidget *menu_item;
+	gchar *date_string;
+	time_t now_raw;
+	struct tm* now;
+	option_menu = glade_xml_get_widget (xml_dialog,
+					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET);
+	menu = gtk_menu_new ();
+
+	now_raw = time (NULL);
+	now = localtime (&now_raw);
+
+	date_string = eel_strdup_strftime ("%c", now);
+	menu_item = gtk_menu_item_new_with_label (date_string);
+	g_free (date_string);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
+	date_string = eel_strdup_strftime ("%Y-%m-%d %H:%M:%S", now);
+	menu_item = gtk_menu_item_new_with_label (date_string);
+	g_free (date_string);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
+	date_string = eel_strdup_strftime (_("today at %-I:%M:%S %p"), now);
+	menu_item = gtk_menu_item_new_with_label (date_string);
+	g_free (date_string);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
+}
+
 static void
 set_columns_from_gconf (NautilusColumnChooser *chooser)
 {
@@ -522,6 +571,7 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 	nautilus_file_management_properties_size_group_create (xml_dialog,
 							       "preview_label",
 							       5);
+	create_date_format_menu (xml_dialog);
 
 	/* setup preferences */
 	eel_preferences_glade_connect_bool (xml_dialog,
@@ -533,6 +583,9 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 	eel_preferences_glade_connect_bool (xml_dialog,
 					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_FOLDERS_FIRST_WIDGET,
 					    NAUTILUS_PREFERENCES_SORT_DIRECTORIES_FIRST); 
+	eel_preferences_glade_connect_bool (xml_dialog,
+					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET,
+					    NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
 	eel_preferences_glade_connect_bool (xml_dialog,
 					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET,
 					    NAUTILUS_PREFERENCES_CONFIRM_TRASH);
@@ -552,52 +605,56 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DEFAULT_VIEW_WIDGET,
 							       NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER,
-							       default_view_values);
+							       (const char **) default_view_values);
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ICON_VIEW_ZOOM_WIDGET,						     
 							       NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
-							       zoom_values);
+							       (const char **) zoom_values);
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_LIST_VIEW_ZOOM_WIDGET,
 							       NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
-							       zoom_values);
+							       (const char **) zoom_values);
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET,
 							       NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_ORDER,
-							       sort_order_values);
+							       (const char **) sort_order_values);
 	eel_preferences_glade_connect_string_enum_option_menu_slave (xml_dialog,
 								     NAUTILUS_FILE_MANAGEMENT_PROPERTIES_SORT_ORDER_WIDGET,
 								     NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER);
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_TEXT_WIDGET,
 							       NAUTILUS_PREFERENCES_SHOW_TEXT_IN_ICONS,
-							       preview_values);
+							       (const char **) preview_values);
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_IMAGE_WIDGET,
 							       NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
-							       preview_values);
+							       (const char **) preview_values);
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_SOUND_WIDGET,
 							       NAUTILUS_PREFERENCES_PREVIEW_SOUND,
-							       preview_values);
+							       (const char **) preview_values);
 	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
 							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_PREVIEW_FOLDER_WIDGET,
 							       NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
-							       preview_values);
+							       (const char **) preview_values);
+	eel_preferences_glade_connect_string_enum_option_menu (xml_dialog,
+							       NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DATE_FORMAT_WIDGET,
+							       NAUTILUS_PREFERENCES_DATE_FORMAT,
+							       (const char **) date_format_values);
 
 	eel_preferences_glade_connect_string_enum_radio_button (xml_dialog,
-								click_behavior_components,
+								(const char **) click_behavior_components,
 								NAUTILUS_PREFERENCES_CLICK_POLICY,
-								click_behavior_values);
+								(const char **) click_behavior_values);
 	eel_preferences_glade_connect_string_enum_radio_button (xml_dialog,
-								executable_text_components,
+								(const char **) executable_text_components,
 								NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
-								executable_text_values);
+								(const char **) executable_text_values);
 
 	eel_preferences_glade_connect_int_enum (xml_dialog,
 						NAUTILUS_FILE_MANAGEMENT_PROPERTIES_THUMBNAIL_LIMIT_WIDGET,
 						NAUTILUS_PREFERENCES_IMAGE_FILE_THUMBNAIL_LIMIT,
-						thumbnail_limit_values);
+						(const int *) thumbnail_limit_values);
 
 
 	nautilus_file_management_properties_dialog_setup_icon_caption_page (xml_dialog);
