@@ -174,6 +174,8 @@ static void     nautilus_property_browser_hide_callback         (GtkWidget
 								 gpointer                       user_data);
 static void     nautilus_property_browser_drag_end              (GtkWidget                     *widget,
 								 GdkDragContext                *context);
+static void     nautilus_property_browser_drag_begin            (GtkWidget                     *widget,
+								 GdkDragContext                *context);
 static void     nautilus_property_browser_drag_data_get         (GtkWidget                     *widget,
 								 GdkDragContext                *context,
 								 GtkSelectionData              *selection_data,
@@ -191,6 +193,12 @@ static void     element_clicked_callback                        (GtkWidget
 								 const EelImageTableEvent *event,
 								 gpointer                       callback_data);
 
+static GdkPixbuf * make_drag_image                              (NautilusPropertyBrowser       *property_browser,
+								 const char                    *file_name);
+static GdkPixbuf * make_color_drag_image                        (NautilusPropertyBrowser       *property_browser,
+								 const char                    *color_spec,
+								 gboolean                       trim_edges);
+
 
 #define BROWSER_CATEGORIES_FILE_NAME "browser.xml"
 
@@ -232,6 +240,7 @@ nautilus_property_browser_class_init (GtkObjectClass *object_klass)
 	klass = NAUTILUS_PROPERTY_BROWSER_CLASS (object_klass);
 
 	object_klass->destroy = nautilus_property_browser_destroy;
+	widget_class->drag_begin = nautilus_property_browser_drag_begin;
 	widget_class->drag_data_get  = nautilus_property_browser_drag_data_get;
 	widget_class->drag_end  = nautilus_property_browser_drag_end;
 }
@@ -561,6 +570,53 @@ nautilus_property_browser_set_drag_type (NautilusPropertyBrowser *property_brows
 	property_browser->details->drag_type = g_strdup (new_drag_type);
 }
 
+static void
+nautilus_property_browser_drag_begin (GtkWidget *widget,
+				      GdkDragContext *context)
+{
+	NautilusPropertyBrowser *property_browser;
+	GtkWidget *child;
+	GdkPixbuf *pixbuf;
+	int x_delta, y_delta;
+	char *element_name;
+
+	property_browser = NAUTILUS_PROPERTY_BROWSER (widget);
+
+	child = g_object_steal_data (G_OBJECT (property_browser), "dragged-image");
+	g_return_if_fail (child != NULL);
+
+	element_name = g_object_get_data (G_OBJECT (child), "property-name");
+	g_return_if_fail (child != NULL);
+
+	/* compute the offsets for dragging */
+	if (strcmp (drag_types[0].target, "application/x-color") != 0) {
+		/* it's not a color, so, for now, it must be an image */
+		/* fiddle with the category to handle the "reset" case properly */
+		char * save_category = property_browser->details->category;
+		if (eel_strcmp (property_browser->details->category, "colors") == 0) {
+			property_browser->details->category = "patterns";
+		}
+		pixbuf = make_drag_image (property_browser, element_name);
+		property_browser->details->category = save_category;
+	} else {
+		pixbuf = make_color_drag_image (property_browser, element_name, TRUE);
+	}
+
+        /* set the pixmap and mask for dragging */       
+	if (pixbuf != NULL) {
+		x_delta = gdk_pixbuf_get_width (pixbuf) / 2;
+		y_delta = gdk_pixbuf_get_height (pixbuf) / 2;
+
+		gtk_drag_set_icon_pixbuf
+			(context,
+			 pixbuf,
+			 x_delta, y_delta);
+		g_object_unref (pixbuf);
+	}
+
+}
+
+
 /* drag and drop data get handler */
 
 static void
@@ -1502,10 +1558,7 @@ element_clicked_callback (GtkWidget *image_table,
 	NautilusPropertyBrowser *property_browser;
 	GtkTargetList *target_list;	
 	GdkDragContext *context;
-	GdkPixbuf *pixbuf;
-	int x_delta, y_delta;
 	const char *element_name;
-	int scroll_offset_x, scroll_offset_y;
 	GdkDragAction action;
 
 	g_return_if_fail (EEL_IS_IMAGE_TABLE (image_table));
@@ -1537,6 +1590,8 @@ element_clicked_callback (GtkWidget *image_table,
 	target_list = gtk_target_list_new (drag_types, G_N_ELEMENTS (drag_types));
 	nautilus_property_browser_set_dragged_file(property_browser, element_name);
 	action = event->button == 3 ? GDK_ACTION_ASK : GDK_ACTION_MOVE | GDK_ACTION_COPY;
+
+	g_object_set_data (G_OBJECT (property_browser), "dragged-image", child);
 	
 	context = gtk_drag_begin (GTK_WIDGET (property_browser),
 				  target_list,
@@ -1544,37 +1599,7 @@ element_clicked_callback (GtkWidget *image_table,
 				  event->button,
 				  event->event);
 	gtk_target_list_unref (target_list);
-	
-	/* compute the offsets for dragging */
-	if (strcmp (drag_types[0].target, "application/x-color")) {
-		/* it's not a color, so, for now, it must be an image */
-		/* fiddle with the category to handle the "reset" case properly */
-		char * save_category = property_browser->details->category;
-		if (eel_strcmp (property_browser->details->category, "colors") == 0) {
-			property_browser->details->category = "patterns";
-		}
-		pixbuf = make_drag_image (property_browser, element_name);
-		property_browser->details->category = save_category;
-		x_delta = gdk_pixbuf_get_width (pixbuf) / 2;
-		y_delta = gdk_pixbuf_get_height (pixbuf) / 2;
-	} else {
-		pixbuf = make_color_drag_image (property_browser, element_name, TRUE);
-		gdk_window_get_position (GTK_VIEWPORT (image_table->parent)->bin_window,
-					 &scroll_offset_x,
-					 &scroll_offset_y);
-		x_delta = event->x - child->allocation.x - scroll_offset_x;
-		y_delta = event->y - child->allocation.y - scroll_offset_y;
-	}
 
-        /* set the pixmap and mask for dragging */       
-	if (pixbuf != NULL) {
-		gtk_drag_set_icon_pixbuf
-			(context,
-			 pixbuf,
-			 x_delta, y_delta);
-		g_object_unref (pixbuf);
-	}
-	
 	/* optionally (if the shift key is down) hide the property browser - it will later be destroyed when the drag ends */	
 	property_browser->details->keep_around = (event->state & GDK_SHIFT_MASK) == 0;
 	if (! property_browser->details->keep_around) {
@@ -1788,7 +1813,7 @@ add_reset_property (NautilusPropertyBrowser *property_browser)
 	
 	g_free (reset_image_file_name);
 
-	reset_image = labeled_image_new (NULL, reset_chit, RESET_IMAGE_NAME, PANGO_SCALE_MEDIUM);
+	reset_image = labeled_image_new ("Reset", reset_chit, RESET_IMAGE_NAME, PANGO_SCALE_MEDIUM);
 	gtk_container_add (GTK_CONTAINER (property_browser->details->content_table), reset_image);
 	eel_wrap_table_reorder_child (EEL_WRAP_TABLE (property_browser->details->content_table),
 					   reset_image,
