@@ -73,9 +73,11 @@ static void                update_icon                             (NautilusSide
 static GtkWidget *         sidebar_title_create_title_label        (void);
 static GtkWidget *         sidebar_title_create_more_info_label    (void);
 static void		   update_all 				   (NautilusSidebarTitle      *sidebar_title);
+static void		   update_more_info			   (NautilusSidebarTitle      *sidebar_title);
 static void		   update_title_font			   (NautilusSidebarTitle      *sidebar_title);
 static void                style_set                               (GtkWidget                 *widget,
 								    GtkStyle                  *previous_style);
+static guint		   get_best_icon_size 			   (NautilusSidebarTitle      *sidebar_title);
 
 struct NautilusSidebarTitleDetails {
 	NautilusFile		*file;
@@ -171,11 +173,17 @@ nautilus_sidebar_title_init (NautilusSidebarTitle *sidebar_title)
 	gtk_widget_show (sidebar_title->details->notes);
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->notes, 0, 0, 0);
 
+	sidebar_title->details->best_icon_size = get_best_icon_size (sidebar_title);
 	/* Keep track of changes in graphics trade offs */
 	update_all (sidebar_title);
 
 	/* initialize the label colors & fonts */
 	style_set (GTK_WIDGET (sidebar_title), NULL);
+
+	eel_preferences_add_callback_while_alive (
+		NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+		(EelPreferencesCallback) update_more_info,
+		sidebar_title, G_OBJECT (sidebar_title));
 }
 
 /* destroy by throwing away private storage */
@@ -673,6 +681,9 @@ sidebar_title_create_title_label (void)
 	eel_gtk_label_make_bold (GTK_LABEL (title_label));
 	gtk_label_set_line_wrap (GTK_LABEL (title_label), TRUE);
 	gtk_label_set_justify (GTK_LABEL (title_label), GTK_JUSTIFY_CENTER);
+	gtk_label_set_selectable (GTK_LABEL (title_label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (title_label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (title_label), 0., 0.5);
 
 	return title_label;
 }
@@ -685,6 +696,9 @@ sidebar_title_create_more_info_label (void)
 	more_info_label = gtk_label_new ("");
 	eel_gtk_label_set_scale (GTK_LABEL (more_info_label), PANGO_SCALE_SMALL);
 	gtk_label_set_justify (GTK_LABEL (more_info_label), GTK_JUSTIFY_CENTER);
+	gtk_label_set_selectable (GTK_LABEL (more_info_label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (more_info_label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (more_info_label), 0., 0.5);
 	
 	return more_info_label;
 }
