@@ -70,6 +70,7 @@ struct NautilusWindowDetails
 	char *title;
 	NautilusFile *viewed_file;
         gboolean viewed_file_seen;
+	gboolean allow_stop;
 
         /* New location. */
         NautilusLocationChangeType location_change_type;
@@ -101,6 +102,7 @@ struct _NautilusNavigationWindowDetails {
         
         /* Location bar */
         gboolean temporary_navigation_bar;
+        gboolean temporary_location_bar;
 
         /* Side Pane */
         int side_pane_width;
